# Copyright (C) 2007 by Intevation GmbH
# Authors:
# Bernhard Herzog <bh@intevation.de>
#
# This program is free software under the GPL (>=v2)
# Read the file COPYING coming with the software for details.

"""Classes to modify HTTP requests."""

import urlparse
from urllib import quote_plus
from StringIO import StringIO
from ConfigParser import SafeConfigParser

from lxml import etree

import getpassword


class IdentityTranscoder(object):

    """A transcoder that does not change anything

    Other transcoders can be derived from this class so that they only
    need to implement the methods that actually change parts of a
    request.
    """

    def __init__(self, method, spliturl):
        """Initializes the transcoder

        Parameters:
        method -- The HTTP method, either 'GET' or 'POST'
        spliturl -- A tuple with the split url of the real remote host.
                    The tuple has the form of the returl value of
                    urlparse.urlsplit
        """
        self.method = method
        self.spliturl = spliturl

    def get_password(self):
        """Returns the username and password needed for the request

        The default implementation here simply uses the host and path
        parts of the url as the parameter for the
        get_password_with_cache function.
        """
        return getpassword.get_password_with_cache("".join(self.spliturl[1:3]))

    def get_url(self):
        """Returns the real url to connect to when handling this request

        The default implementation simply returns the recombined url
        given to the constructor
        """
        return urlparse.urlunsplit(self.spliturl)

    def convert_body(self, content_type, body):
        """Convert the body of the request

        Parameters:
        content_type -- The value of the Content-Type header of the request.
                        If no Content-Type was give, the value is None
        body -- The body of the request as a string.

        The return value is a tuple (new_content_type, new_body).  The
        default implementation simply returns the parameters unchanged.
        """
        return content_type, body


class HTTPSTranscoder(IdentityTranscoder):

    """Transcoder that turns the URL unconditionally into a HTTPS URL"""

    def __init__(self, method, spliturl):
        super(HTTPSTranscoder, self).__init__(method, ("https",) + spliturl[1:])


class OWSProxyGETTranscoder(HTTPSTranscoder):

    """Transcoder for GET requests to the OWSProxy"""

    def get_url(self):
        """Returns the URL with username and password added to the query"""
        scheme, netloc, path, query, fragment = self.spliturl
        if self.method == "GET":
            user, password = self.get_password()
            # FIXME: what do we do if the user cancelled?
            if user is not None:
                if query:
                    split_query = query.split("&")
                else:
                    split_query = []
                query = "&".join(split_query
                                 + ["user=%s" % quote_plus(user),
                                    "password=%s" % quote_plus(password)])
        return urlparse.urlunsplit((scheme, netloc, path, query, fragment))


class OWSProxyPOSTTranscoder(HTTPSTranscoder):

    """Transcoder for POST requests to the OWSProxy"""

    def convert_body(self, content_type, body):
        """Adds user and password attributes to the GetFeature element.
        """
        t = etree.parse(StringIO(body))
        root = t.getroot()
        if root.tag in ("{http://www.opengis.net/wfs}GetFeature",
                        "{http://www.opengis.net/wfs}Transaction"):
            user, password = self.get_password()
            root.set("user", user)
            root.set("password", password)
            body = etree.tostring(t, xml_declaration=True)
            content_type = "text/xml"
        return content_type, body


class TranscoderMap(object):

    """Manages the host specific transcoder settings"""

    def __init__(self, classes):
        """Initializes the TranscoderMap.

        The parameter 'classes' should be a sequence of (name, get,
        post) triples.  They will be used to add classes with the
        add_class method.  The first item in classes will become the
        default class.
        """
        self.hostmap = dict()
        self.classmap = dict()
        self.default_classname = None

        if classes:
            for item in classes:
                self.add_class(*item)
            self.set_default_class(classes[0][0])

    def set_default_class(self, name):
        """Sets the default class"""
        self.default_classname = name

    def add_class(self, name, get, post):
        """Adds a transcoder class named name.
        The get and post parameters are the python classes implementing
        the transcoder for the GET and POST methods.
        """
        self.classmap[name] = dict(GET=get, POST=post)

    def add_host(self, host, path, classname):
        """Adds a host description.

        A connection path on host will be done with the appropriate
        transcoder from the class given by classname.
        """
        self.hostmap[(host, path)] = classname

    def lookup(self, method, host, path):
        """Returns the python class implementing the transcoder for path on host
        """
        classname = self.hostmap.get((host, path), self.default_classname)
        return self.classmap[classname][method]

    def get_transcoder(self, method, url):
        """Returns a transcoder for the given HTTP method and URL

        URL should be the URL given for the GET or POST http request.
        This function extracts the real remote host from the URL and
        uses the URL to look up the transcoder with lookup_transcoder.

        This method supports two ways to encode the remote host in the
        URL:

           -- InteProxy is accessed directly as an HTTP server.  In this
              cases URL is an absolute pathname and does not include a
              hostname in the normal URL syntax.  Instead get_transcoder
              assumes that it encodes the real remote host in the first
              component of the path.  This is the old InteProxy method

           -- InteProxy is used as a HTTP-proxy.  In this case URL will
              be a full URL including a hostname.  In this case
              get_transcoder will use the host of the URL as the real
              remote host.  The path component of the URL is not
              modified.
        """
        scheme, netloc, path, query, fragment = urlparse.urlsplit(url)

        # If scheme is given, InteProxy is used as a HTTP proxy, otherwise
        # it's the old InteProxy approach.  Only the old approach needs
        # special casing.
        if not scheme:
            scheme = "http"
            # determine the remote host encoded in the path
            split_path = path.split("/")
            if not split_path[0]:
                del split_path[0]
            netloc = split_path.pop(0)
            path = "/" + "/".join(split_path)

        transcoder_class = self.lookup(method, netloc, path)
        return transcoder_class(method,
                                (scheme, netloc, path, query, fragment))

    def read_config(self, filename):
        """Reads a configuration file and adds the host descriptions found"""
        parser = SafeConfigParser()
        parser.read([filename])
        for section in parser.sections():
            host = parser.get(section, "host")
            path = parser.get(section, "path")
            cls = parser.get(section, "class")
            self.add_host(host, path, cls)


def create_transcoder_map():
    return TranscoderMap([
        ("identity", IdentityTranscoder, IdentityTranscoder),
        ("owsproxy", OWSProxyGETTranscoder, OWSProxyPOSTTranscoder),
        ])
transcoder_map = create_transcoder_map()
