# Copyright (C) 2007 by Intevation GmbH
# Authors:
# Bernhard Herzog <bh@intevation.de>
#
# This program is free software under the GPL (>=v2)
# Read the file COPYING coming with the software for details.

"""Tests for TranscoderMap"""

import unittest

import support

from inteproxy.transcoder import create_transcoder_map, \
     OWSProxyGETTranscoder, OWSProxyPOSTTranscoder, IdentityTranscoder


class TestTranscoderMap(unittest.TestCase, support.FileTestMixin):

    config_contents = """\
[inteproxy-demo.intevation.org]
host=inteproxy-demo.intevation.org
path=/cgi-bin/frida-wms
class=owsproxy

[example.com]
host=example.com
path=/foo
class=owsproxy
"""
    #

    def setUp(self):
        self.config_filename = self.create_temp_file(self.id(),
                                                     self.config_contents)

    def test_lookup(self):
        """Test the lookup method"""
        transcoder_map = create_transcoder_map()
        transcoder_map.read_config(self.config_filename)

        test_cases = [
            ("GET", "inteproxy-demo.intevation.org", "/cgi-bin/frida-wms",
             OWSProxyGETTranscoder),
            ("POST", "inteproxy-demo.intevation.org", "/cgi-bin/frida-wms",
             OWSProxyPOSTTranscoder),
            ("GET", "inteproxy-demo.intevation.org", "/cgi-bin/frida",
             IdentityTranscoder),
            ("POST", "inteproxy-demo.intevation.org", "/cgi-bin/frida",
             IdentityTranscoder),

            ("GET", "example.com", "/foo",
             OWSProxyGETTranscoder),
            ("POST", "example.com", "/foo",
             OWSProxyPOSTTranscoder),

            # Example for a non-WMS request sent by deejump
            ("GET", "schemas.opengeospatial.net",
             "/wms/1.1.1/WMS_MS_Capabilities.dtd", IdentityTranscoder),
            ]

        for method, host, path, expectedcls in test_cases:
            cls = transcoder_map.lookup(method, host, path)
            msg=("Wrong class for %(method)s %(host)s%(path)s:"
                 " got %(cls)s expected %(expectedcls)s" % locals())
            self.assertEquals(cls, expectedcls, msg)

    def test_get_transcoder_http_proxy(self):
        """Test the get_transcoder method in http-proxy mode"""
        transcoder_map = create_transcoder_map()
        transcoder_map.read_config(self.config_filename)

        test_cases = [
            ("GET", "http://inteproxy-demo.intevation.org/cgi-bin/frida-wms",
             OWSProxyGETTranscoder),
            ("POST", "http://inteproxy-demo.intevation.org/cgi-bin/frida-wms",
             OWSProxyPOSTTranscoder),
            ("GET", "http://inteproxy-demo.intevation.org/cgi-bin/frida",
             IdentityTranscoder),
            ("POST", "http://inteproxy-demo.intevation.org/cgi-bin/frida",
             IdentityTranscoder),

            ("GET", "http://example.com/foo", OWSProxyGETTranscoder),
            ("POST", "http://example.com/foo", OWSProxyPOSTTranscoder),

            # Example for a non-WMS request sent by deejump
            ("GET",
        "http://schemas.opengeospatial.net/wms/1.1.1/WMS_MS_Capabilities.dtd",
             IdentityTranscoder),
            ]

        for method, url, expectedcls in test_cases:
            transcoder = transcoder_map.get_transcoder(method, url)
            cls = transcoder.__class__
            msg=("Wrong class for %(method)s %(url)s:"
                 " got %(cls)s expected %(expectedcls)s" % locals())
            self.assertEquals(cls, expectedcls, msg)

    def test_get_transcoder_inteproxy_style(self):
        """Test the get_transcoder method in InteProxy mode"""
        transcoder_map = create_transcoder_map()
        transcoder_map.read_config(self.config_filename)

        test_cases = [
            ("GET", "/inteproxy-demo.intevation.org/cgi-bin/frida-wms",
             OWSProxyGETTranscoder),
            ("POST", "/inteproxy-demo.intevation.org/cgi-bin/frida-wms",
             OWSProxyPOSTTranscoder),
            ("GET", "/inteproxy-demo.intevation.org/cgi-bin/frida",
             IdentityTranscoder),
            ("POST", "/inteproxy-demo.intevation.org/cgi-bin/frida",
             IdentityTranscoder),

            ("GET", "/example.com/foo", OWSProxyGETTranscoder),
            ("POST", "/example.com/foo", OWSProxyPOSTTranscoder),
            ]

        for method, url, expectedcls in test_cases:
            transcoder = transcoder_map.get_transcoder(method, url)
            cls = transcoder.__class__
            msg=("Wrong class for %(method)s %(url)s:"
                 " got %(cls)s expected %(expectedcls)s" % locals())
            self.assertEquals(cls, expectedcls, msg)
